from datetime import datetime


def string_to_datetime(date_string):
    """
    Convert a date string to a datetime object.

    This function supports the following formats:
    - ISO 8601 with and without timezone offset: e.g., '2023-08-02T09:27:54-05:00' or '2023-08-07T18:32:05.537Z'
    - RFC 1123: e.g., 'Mon, 07 Aug 2023 18:32:05 GMT'

    Args:
    - date_string (str): The date string to convert

    Returns:
    - datetime.datetime: The datetime representation of the provided date string.

    Raises:
    - ValueError: If the provided date string doesn't match any of the supported formats.
    """
    try:
        # First, try ISO 8601 format
        if "Z" in date_string:
            return datetime.fromisoformat(date_string.replace("Z", "+00:00"))
        return datetime.fromisoformat(date_string)
    except ValueError:
        pass

    try:
        # Then, try RFC 1123 format
        return datetime.strptime(date_string, "%a, %d %b %Y %H:%M:%S %Z")
    except ValueError:
        pass

    # If neither format matches, raise an exception
    raise ValueError(f"Unsupported date format: {date_string}")
